==> trilinos_12.18.1.orig.tar.gz: Trilinos-trilinos-release-12-18-1/packages/muelu/src/Graph/Containers/MueLu_LinkedList.cpp <==
// @HEADER
//
// ***********************************************************************
//
//        MueLu: A package for multigrid based preconditioning
//                  Copyright 2012 Sandia Corporation
//
// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
// the U.S. Government retains certain rights in this software.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the Corporation nor the names of the
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Questions? Contact
//                    Jonathan Hu       (jhu@sandia.gov)
//                    Andrey Prokopenko (aprokop@sandia.gov)
//                    Ray Tuminaro      (rstumin@sandia.gov)
//
// ***********************************************************************
//
// @HEADER
#include <stddef.h>                     // for NULL
#include "MueLu_LinkedList.hpp"

namespace MueLu {

  LinkedList::LinkedList() : nodeHead(NULL), nodeTail(NULL) { }

  LinkedList::~LinkedList() {
    while (nodeHead != NULL)
      DeleteHead();
  }

  bool LinkedList::IsEmpty() {
    return nodeHead == NULL;
  }

  void LinkedList::Add(int iNode) {
    MueLu_Node *newNode = new MueLu_Node;
    newNode->nodeId = iNode;
    newNode->next = NULL;
    if (nodeHead == NULL) {
      nodeHead = newNode;
      nodeTail = newNode;
    } else {
      nodeTail->next = newNode;
      nodeTail = newNode;
    }
  }

  int LinkedList::Pop() { // get head and remove first node
    if (IsEmpty()) return -1;

    int iNode = nodeHead->nodeId;
    DeleteHead();
    return iNode;
  }

  void LinkedList::DeleteHead() {
    if (IsEmpty()) return;

    MueLu_Node *newNode = nodeHead;
    nodeHead = newNode->next;
    delete newNode;
  }

}

//TODO: nodeTail unused -> remove?

==> trilinos-15.1.1.tar.gz: Trilinos-trilinos-release-15-1-1/packages/muelu/src/Graph/Containers/MueLu_LinkedList.cpp <==
// @HEADER
//
// ***********************************************************************
//
//        MueLu: A package for multigrid based preconditioning
//                  Copyright 2012 Sandia Corporation
//
// Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
// the U.S. Government retains certain rights in this software.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// 3. Neither the name of the Corporation nor the names of the
// contributors may be used to endorse or promote products derived from
// this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY SANDIA CORPORATION "AS IS" AND ANY
// EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SANDIA CORPORATION OR THE
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// Questions? Contact
//                    Jonathan Hu       (jhu@sandia.gov)
//                    Andrey Prokopenko (aprokop@sandia.gov)
//                    Ray Tuminaro      (rstumin@sandia.gov)
//
// ***********************************************************************
//
// @HEADER
#include <stddef.h>  // for NULL
#include "MueLu_LinkedList.hpp"

namespace MueLu {

LinkedList::LinkedList()
  : nodeHead(NULL)
  , nodeTail(NULL) {}

LinkedList::~LinkedList() {
  while (nodeHead != NULL)
    DeleteHead();
}

bool LinkedList::IsEmpty() {
  return nodeHead == NULL;
}

void LinkedList::Add(int iNode) {
  MueLu_Node *newNode = new MueLu_Node;
  newNode->nodeId     = iNode;
  newNode->next       = NULL;
  if (nodeHead == NULL) {
    nodeHead = newNode;
    nodeTail = newNode;
  } else {
    nodeTail->next = newNode;
    nodeTail       = newNode;
  }
}

int LinkedList::Pop() {  // get head and remove first node
  if (IsEmpty()) return -1;

  int iNode = nodeHead->nodeId;
  DeleteHead();
  return iNode;
}

void LinkedList::DeleteHead() {
  if (IsEmpty()) return;

  MueLu_Node *newNode = nodeHead;
  nodeHead            = newNode->next;
  delete newNode;
}

}  // namespace MueLu

// TODO: nodeTail unused -> remove?
